package com.kdm.erraiexample.client;

import java.util.List;
import java.util.logging.Handler;
import java.util.logging.Level;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.jboss.errai.common.client.api.Caller;
import org.jboss.errai.common.client.logging.LoggingHandlerConfigurator;
import org.jboss.errai.common.client.logging.handlers.ErraiConsoleLogHandler;
import org.jboss.errai.enterprise.client.jaxrs.MarshallingWrapper;
import org.jboss.errai.enterprise.client.jaxrs.api.ResponseCallback;
import org.jboss.errai.enterprise.client.jaxrs.api.RestClient;
import org.jboss.errai.ioc.client.api.EntryPoint;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gwt.http.client.Response;
import com.kdm.erraiexample.shared.SearchService;
import com.kdm.provider.data.Detail;
import com.kdm.provider.data.Facet;
import com.kdm.provider.data.SearchResult;
import com.vividsolutions.jts.geom.Geometry;

@EntryPoint
public class ErraiExampleApp
{
  private static final Logger log = LoggerFactory.getLogger(ErraiExampleApp.class);
  
  @Inject
  private Caller<SearchService> searchService;
  
  
  @PostConstruct
  private void init()
  {
    logConfig();
    RestClient.setApplicationRoot("/erraiexample/service");
    RestClient.setJacksonMarshallingActive(true);
    log.info("ErraiExampleApp:  init.");
    // Have to perform a search when the page loads.
    // This callback will receive the search results and  
    // set the result values to this form
    final ResponseCallback searchResultCallback = new ResponseCallback()
    {
      @Override
      public void callback(Response response)
      {
        log.info("searchResultCallback: callback: setting up, response.getStatusCode()="
            + response.getStatusCode());
        if (response.getStatusCode() == 200)
        {
          log.info("searchResultCallback:  service response text:'" + response.getText() + "'");
          response.getClass();
          SearchResult re = MarshallingWrapper.fromJSON(response.getText(), SearchResult.class);
          log.info("searchResultCallback:  de-marshalled!");
          
          List<Detail> details = re.getDetails();
          for (Detail detail : details) {
            log.info("searchResultCallback:   detail.value:'" +  detail.getValue() + "'");
          }
          Geometry geoShape = re.getGeoShape();
          int numGeometries = geoShape.getNumGeometries();
          log.info("searchResultCallback:  numGeometries="+numGeometries);
          log.info("searchResultCallback:  about to reference Facets:");
          
          List<Facet> facets = re.getFacets();
          log.info("searchResultCallback:  Facets:1");
          for (Facet facet : facets) {
            log.info("searchResultCallback:   facet.count:'" +  facet.count + "'");
//            List<FacetEntry> facetEntries = facet.entries;
//          log.info("searchResultCallback:   facet.count:'" +  pojo.getValue() + "'");
//            facetEntryListWidget.setItems(facetEntries);
          }

          log.info("searchResultCallback:  done.");
        }
      }
    };
  // Create a query and run 
  searchService.call(searchResultCallback).search("a search string");
    
  }
  
  private void logConfig()
  {
    LoggingHandlerConfigurator config = LoggingHandlerConfigurator.get();
    Handler handler = config.getHandler(ErraiConsoleLogHandler.class);
    handler.setLevel(Level.ALL);
    
    log.debug("Should now see debug-level messages?!");
  }
  
}