package com.kdm.erraiexample.client.marshaller;

import org.jboss.errai.marshalling.client.api.MarshallingSession;
import org.jboss.errai.marshalling.client.api.annotations.ClientMarshaller;
import org.jboss.errai.marshalling.client.api.json.EJValue;
import org.jboss.errai.marshalling.client.marshallers.AbstractNullableMarshaller;

import com.vividsolutions.jts.geom.Geometry;

/**
 * Handles marshalling JTS Geometry on the client-side into geojson to be
 * sent over the wire to our JAX-RS REST service(s).
 */
@ClientMarshaller(Geometry.class)
public class GeometryMarshaller extends AbstractNullableMarshaller<Geometry>
{
  private final GeometryDeserializer deserializer = new GeometryDeserializer();
  private final GeometrySerializer serializer = new GeometrySerializer();
  
  @Override
  public Geometry[] getEmptyArray()
  { return new Geometry[0]; }

  @Override
  public Geometry doNotNullDemarshall(EJValue o, MarshallingSession ctx)
  {
    return deserializer.deserialize(o);
  }

  @Override
  public String doNotNullMarshall(Geometry o, MarshallingSession ctx)
  {
    return serializer.serialize(o);
  }
}
