package com.kdm.erraiexample.client.marshaller;

import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONNumber;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.json.client.JSONString;
import com.google.gwt.json.client.JSONValue;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

/**
 * This class serilizes JTS Geometry objects into geojson.  It has been derived 
 * from the Jackson GeoJSON project:
 *
 * https://github.com/bedatadriven/jackson-geojson/blob/master/src/main/java/com/bedatadriven/geojson/jackson2/GeometrySerializer.java
 */
public class GeometrySerializer
{
  public String serialize(Geometry value)
  {
    // TODO: handle other Geometry types; for now we only need Polygon support...
    if (value instanceof Polygon)
    {
      return writePolygon((Polygon)value);
    }
    else
    {
      throw new UnsupportedOperationException("Unsupported Geometry type: " + value.getClass().getName());
    }
  }

  private String writePolygon(Polygon value)
  {
    JSONObject obj = new JSONObject();
    obj.put("type", new JSONString("Polygon"));
    obj.put("coordinates", writePolygonCoordinates(value));
    return obj.toString();
  }

  private JSONValue writePolygonCoordinates(Polygon value)
  {
    JSONArray array = new JSONArray();
    array.set(0, writeLineStringCoords(value.getExteriorRing()));

    for (int i = 0; i != value.getNumInteriorRing(); ++i) 
    {
      array.set(i, writeLineStringCoords(value.getInteriorRingN(i)));
    }
    return array;
  }

  private JSONArray writeLineStringCoords(LineString ring)
  {
    JSONArray array = new JSONArray();
    for (int i = 0; i != ring.getNumPoints(); ++i) 
    {
      Point p = ring.getPointN(i);
      array.set(i, writePointCoords(p));
    }
    return array;
  }

  private JSONArray writePointCoords(Point p)
  {
    JSONArray array = new JSONArray();
    array.set(0, new JSONNumber(p.getX()));
    array.set(1, new JSONNumber(p.getY()));
    return array;
  }
}
