package com.kdm.erraiexample.config;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.ext.Provider;

import org.apache.log4j.Logger;
import org.codehaus.jackson.map.ObjectMapper;
import org.jboss.resteasy.plugins.providers.jackson.ResteasyJacksonProvider;

import com.bedatadriven.geojson.GeoJsonModule;

/**
 * This Provider configures JBoss7's built-in JAX-RS implementation to handle 
 * serializing/de-serializing GeoJSON objects as JTS Geometry objects.  For this
 * to work we also need to add the following "Dependencies" entry to our .war 
 * manifest:
 * 
 * org.jboss.resteasy.resteasy-jackson-provider,
 * org.codehaus.jackson.jackson-jaxrs,
 * org.codehaus.jackson.jackson-mapper-asl,
 * org.codehaus.jackson.jackson-core-asl
 * 
 * As documented at https://community.jboss.org/message/635233.
 */
@Provider
@Consumes({ MediaType.APPLICATION_JSON, "text/json" })
@Produces({ MediaType.APPLICATION_JSON, "text/json" })
public class JacksonConfigurator extends ResteasyJacksonProvider
{
  private static final Logger log = Logger.getLogger(JacksonConfigurator.class);
  
  public JacksonConfigurator()
  {
    super();
    log.info("Configuring Jackson REST provider.");
    
    ObjectMapper mapper = new ObjectMapper();
    mapper.registerModule(new GeoJsonModule());
    setMapper(mapper);
  }
}
